/**
 * 
 */
package bloodnamegen;

import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Neoancient
 * 
 * Tracks details of a Clan Bloodname: original Clan, phenotype, date
 * of acquisition by other Clans, etc.
 *
 * Copyright Carl Spain
 * 
 *     This file is part of BloodnameGen.
 *
 *  BloodnameGen is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  BloodnameGen is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BloodnameGen.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
public class Bloodname {
	public static final int P_GENERAL = 0;
	public static final int P_MECHWARRIOR = 1;
	public static final int P_AEROSPACE = 2;
	public static final int P_ELEMENTAL = 3;
	public static final int P_PROTOMECH = 4;
	public static final int P_NAVAL = 5;
	public static final int P_NUM = 6;
	
	public static final String[] phenotypeNames = {
		"General", "MechWarrior", "Aerospace Pilot", "Elemental",
		"ProtoMech Pilot", "Naval Commander"
	};
	
	private String name;
	private String founder;
	private Clan origClan;
	private boolean exclusive;
	private boolean limited;
	private int inactive;
	private int abjured;
	private int reactivated;
	private int startDate;
	private int phenotype;
	ArrayList<Clan> postReavingClans;
	ArrayList<NameAcquired> acquiringClans;
	NameAcquired absorbed;
	
	public Bloodname() {
		name = "";
		founder = "";
		exclusive = false;
		limited = false;
		inactive = 0;
		abjured = 0;
		reactivated = 0;
		startDate = 2807;
		phenotype = P_GENERAL;
		postReavingClans = new ArrayList<Clan>();
		acquiringClans = new ArrayList<NameAcquired>();
		absorbed = null;
	}
	
	public String getName() {
		return name;
	}

	public String getFounder() {
		return founder;
	}

	public Clan getOrigClan() {
		return origClan;
	}

	public boolean isExclusive() {
		return exclusive;
	}
	
	public boolean isLimited() {
		return limited;
	}

	public boolean isInactive(int year) {
		return year < startDate || (inactive > 0 && inactive < year &&
				!(reactivated > 0 && reactivated <= year));
	}

	public boolean isAbjured(int year) {
		return abjured > 0 && abjured < year;
	}

	public int getPhenotype() {
		return phenotype;
	}

	public ArrayList<Clan> getPostReavingClans() {
		return postReavingClans;
	}

	public ArrayList<NameAcquired> getAcquiringClans() {
		return acquiringClans;
	}

	public NameAcquired getAbsorbed() {
		return absorbed;
	}

	public int phenotypeMultiplier(int warriorType, int year) {
		switch (phenotype) {
		case P_MECHWARRIOR:
			return (warriorType == P_MECHWARRIOR)?3:0;
		case P_AEROSPACE:
			return (warriorType == P_AEROSPACE || warriorType == P_PROTOMECH)?3:0;
		case P_ELEMENTAL:
			if (year < 2870) {
				return 1;
			}
			return (warriorType == P_ELEMENTAL)?3:0;
		case P_PROTOMECH:
			switch (warriorType) {
			case P_PROTOMECH:return 9;
			case P_AEROSPACE:return 1;
			default:return 0;
			}
		case P_NAVAL:
			return (warriorType == P_NAVAL)?3:0;
		case P_GENERAL:
		default:
			return 1;
		}
	}

	public static Bloodname loadFromXml(Node node) {
		Bloodname retVal = new Bloodname();
		NodeList nl = node.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			Node wn = nl.item(i);
			
			if (wn.getNodeName().equalsIgnoreCase("name")) {
				retVal.name = wn.getTextContent().trim();
			} else if (wn.getNodeName().equalsIgnoreCase("founder")) {
				retVal.founder = wn.getTextContent().trim();
			} else if (wn.getNodeName().equalsIgnoreCase("clan")) {
				retVal.origClan = Clan.getClan(wn.getTextContent().trim());
			} else if (wn.getNodeName().equalsIgnoreCase("exclusive")) {
				retVal.exclusive = true;
			} else if (wn.getNodeName().equalsIgnoreCase("reaved")) {
				retVal.inactive = Integer.parseInt(wn.getTextContent().trim());
			} else if (wn.getNodeName().equalsIgnoreCase("dormant")) {
				retVal.inactive = Integer.parseInt(wn.getTextContent().trim()) + 10;
			} else if (wn.getNodeName().equalsIgnoreCase("abjured")) {
				retVal.abjured = Integer.parseInt(wn.getTextContent().trim());
			} else if (wn.getNodeName().equalsIgnoreCase("reactivated")) {
				retVal.reactivated = Integer.parseInt(wn.getTextContent().trim() + 20);
			} else if (wn.getNodeName().equalsIgnoreCase("phenotype")) {
				switch (wn.getTextContent().trim()) {
				case "General":
					retVal.phenotype = P_GENERAL;
					break;
				case "MechWarrior":
					retVal.phenotype = P_MECHWARRIOR;
					break;
				case "Aerospace":
					retVal.phenotype = P_AEROSPACE;
					break;
				case "Elemental":
					retVal.phenotype = P_ELEMENTAL;
					break;
				case "ProtoMech":
					retVal.phenotype = P_PROTOMECH;
					break;
				case "Naval":
					retVal.phenotype = P_NAVAL;
					break;
				default:
					System.err.println("Unknown phenotype " +
							wn.getTextContent() + " in " + retVal.name);
				}
			} else if (wn.getNodeName().equalsIgnoreCase("postReaving")) {
				String[] clans = wn.getTextContent().trim().split(",");
				for (String c : clans) {
					retVal.postReavingClans.add(Clan.getClan(c));
				}
			} else if (wn.getNodeName().equalsIgnoreCase("acquired")) {
				retVal.acquiringClans.add(retVal.new NameAcquired(
						Integer.parseInt(wn.getAttributes().getNamedItem("date").getTextContent()) + 10,
						wn.getTextContent().trim()));
			} else if (wn.getNodeName().equalsIgnoreCase("shared")) {
				retVal.acquiringClans.add(retVal.new NameAcquired(
						Integer.parseInt(wn.getAttributes().getNamedItem("date").getTextContent()),
						wn.getTextContent().trim()));
			} else if (wn.getNodeName().equalsIgnoreCase("absorbed")) {
				retVal.absorbed = retVal.new NameAcquired(
						Integer.parseInt(wn.getAttributes().getNamedItem("date").getTextContent()),
						wn.getTextContent().trim());
			} else if (wn.getNodeName().equalsIgnoreCase("created")) {
				retVal.startDate = Integer.parseInt(wn.getTextContent().trim()) + 20;
			}
		}
		
		return retVal;
	}
	
	public class NameAcquired {
		public int year;
		public String clan;
		public NameAcquired(int y, String c) {
			year = y;
			clan = c;
		}
	}
}
