package bloodnamegen;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import bloodnamegen.Bloodname;
import bloodnamegen.util.Dice;
import bloodnamegen.util.Fraction;

/**
 * @author Neoancient
 * 
 * Selects a Bloodname appropriate to Clan, era, and phenotype.
 *
 * Copyright Carl Spain
 * 
 *     This file is part of BloodnameGen.
 *
 *  BloodnameGen is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  BloodnameGen is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BloodnameGen.  If not, see <http://www.gnu.org/licenses/>.
 */
public class BloodnameSelector implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -6607260964142149900L;
	
	private static ArrayList<Bloodname> bloodnames;
	private static BloodnameSelector bns = null;
	private static boolean initialized = false;
	private static boolean initializing = false;

	private BloodnameSelector() {
	}
	
	public static BloodnameSelector getInstance() {
		if (null == bns) {
			bns = new BloodnameSelector();
			loadFromXml();
		}
		return bns;
	}
	
	public Bloodname randomBloodname(String factionCode, int phenotype, int year) {
		return randomBloodname(Clan.getClan(factionCode), phenotype, year);
	}
	
	public Bloodname randomBloodname(Clan faction, int phenotype, int year) {
		if (Dice.d20() == 1) {
			return randomBloodname(faction.getRivalClan(year), phenotype, year);
		}
		if (Dice.d20() == 2) {
			phenotype = Bloodname.P_GENERAL;
		}
		
		HashMap<Bloodname, Fraction> weights = new HashMap<Bloodname, Fraction>();
		ArrayList<Bloodname> nonExclusives = new ArrayList<Bloodname>();
		double nonExclusivesWeight = 0.0;

		for (Bloodname name : bloodnames) {
			if (name.isInactive(year) ||
					(name.isAbjured(year) && !name.getOrigClan().equals(faction)) ||
					0 == name.phenotypeMultiplier(phenotype, year)) {
				continue;
			}

			Fraction weight = null;

			if (year < 3100) {
				int numClans = 1;
				for (Bloodname.NameAcquired a : name.getAcquiringClans()) {
					if (a.year < year) {
						numClans++;
					}
				}
				if (name.getOrigClan().equals(faction) ||
						(null != name.getAbsorbed() && faction.equals(name.getAbsorbed().clan) &&
						name.getAbsorbed().year > year)) {
					if (name.isExclusive() || numClans > 1) {
						weight = new Fraction(1, numClans);
					} else {
						weight = eraFraction(year);
						nonExclusivesWeight += 1 - eraFraction(year).value();
						weight.mul(eraFraction(year));
					}
				} else {
					for (Bloodname.NameAcquired a : name.getAcquiringClans()) {
						if (faction.equals(a.clan)) {
							weight = new Fraction(1, numClans);
							break;
						}
					}
					if (null == weight && !name.isExclusive()) {
						for (int i = 0; i < name.phenotypeMultiplier(phenotype, year); i++) {
							nonExclusives.add(name);
						}
					}
				}
			} else {
				if (name.getPostReavingClans().contains(faction)) {
					weight = new Fraction(name.phenotypeMultiplier(phenotype, year),
							name.getPostReavingClans().size());
					if (!name.isLimited()) {
						if (name.isExclusive()) {
							weight.mul(4);
						} else {
							weight.mul(2);
						}
					}
				} else if (name.getPostReavingClans().size() == 0) {
					for (int i = 0; i < name.phenotypeMultiplier(phenotype, year); i++) {
						nonExclusives.add(name);
					}
				}
			}
			if (null != weight) {
				weight.mul(name.phenotypeMultiplier(phenotype, year));
				weights.put(name, weight);
			}
		}
		
		int lcd = Fraction.lcd(weights.values());
		for (Fraction f : weights.values()) {
			f.mul(lcd);
		}
		ArrayList<Bloodname> nameList = new ArrayList<Bloodname>();
		for (Bloodname b : weights.keySet()) {
			for (int i = 0; i < weights.get(b).value(); i++) {
				nameList.add(b);
			}
		}
		nonExclusivesWeight *= lcd;
		if (year >= 3100) {
			nonExclusivesWeight = nameList.size() / 10.0;
		}
		int roll = Dice.randomInt(nameList.size() + (int)(nonExclusivesWeight + 0.5));
		if (roll > nameList.size() - 1) {
			return nonExclusives.get(Dice.randomInt(nonExclusives.size()));
		}
		return nameList.get(roll);
	}
	
	private Fraction eraFraction(int year) {
		if (year < 2900) {
			return new Fraction(1);
		}
		if (year < 2950) {
			return new Fraction(9, 10);
		}
		if (year < 3000) {
			return new Fraction(4, 5);
		}
		if (year < 3050) {
			return new Fraction(7, 10);
		}
		return new Fraction (3, 5);
	}
	
	public boolean isInitialized() {
		return initialized;
	}
	
	public boolean isInitializing() {
		return initializing;
	}
	
	private static void loadFromXml() {
		initializing = true;
		bloodnames = new ArrayList<Bloodname>();
		
		File f = new File("data/bloodnames.xml");
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(f);
		} catch (FileNotFoundException e) {
			System.err.println("Cannot find file bloodnames.xml");
			initializing = false;
			return;
		}
		
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document doc = null;
		
		try {
			DocumentBuilder db = dbf.newDocumentBuilder();
			doc = db.parse(fis);
		} catch (Exception ex) {
			System.err.println(ex.getMessage());
		}
		
		Element bloodnameElement = doc.getDocumentElement();
		NodeList nl = bloodnameElement.getChildNodes();
		bloodnameElement.normalize();
		
		for (int i = 0; i < nl.getLength(); i++) {
			Node wn = nl.item(i);
			if (wn.getNodeType() == Node.ELEMENT_NODE) {
				if (wn.getNodeName().equalsIgnoreCase("bloodname")) {
					bloodnames.add(Bloodname.loadFromXml(wn));
				}
			}
		}
		initializing = false;
		initialized = true;
		System.out.println("Loaded " + bloodnames.size() + " Bloodname records.");
	}
}
