/**
 * 
 */
package bloodnamegen;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import bloodnamegen.util.DatedRecord;
import bloodnamegen.util.Dice;

/**
 * @author Neoancient
 * 
 * Tracks information about the various Clans: dates of formation and
 * absorption/annihilation, rivalries, and location (IS/homeworlds) after
 * the Wars of Reaving
 *
 * Copyright Carl Spain
 * 
 *     This file is part of BloodnameGen.
 *
 *  BloodnameGen is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  BloodnameGen is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BloodnameGen.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
public class Clan {
	private static HashMap<String, Clan> allClans;
	
	private String code;
	private String fullName;
	private int startDate;
	private int endDate;
	private int abjurationDate;
	private ArrayList<DatedRecord> rivals;
	private ArrayList<DatedRecord> nameChanges;
	private boolean homeClan;
	
	public Clan() {
		startDate = endDate = abjurationDate = 0;
		rivals = new ArrayList<DatedRecord>();
		nameChanges = new ArrayList<DatedRecord>();
	}
	
	@Override
	public boolean equals(Object o) {
		if (o instanceof Clan) {
			return code.equals(((Clan)o).code);
		}
		if (o instanceof String) {
			return code.equals((String)o);
		}
		return false;
	}
	
	public static Clan getClan(String code) {
		return allClans.get(code);
	}
	
	public String getCode() {
		return code;
	}

	public String getFullName(int year) {
		for (DatedRecord r : nameChanges) {
			if (r.startDate < year &&
					(r.endDate == 0 || r.endDate > year)) {
				return r.descr;
			}
		}
		return fullName;
	}

	public boolean isActive(int year) {
		return startDate < year && (endDate == 0 || endDate > year);
	}

	public boolean isAbjured(int year) {
		if (abjurationDate == 0) return false;
		return abjurationDate < year;
	}

	public ArrayList<Clan> getRivals(int year) {
		ArrayList<Clan> retVal = new ArrayList<Clan>();
		for (DatedRecord r : rivals) {
			if (r.startDate < year && (endDate == 0) || endDate > year) {
				Clan c = allClans.get(r.descr);
				if (c.isActive(year)) {
					retVal.add(c);
				}
			}
		}
		return retVal;
	}

	public boolean isHomeClan() {
		return homeClan;
	}
	
	public Clan getRivalClan(int year) {
		ArrayList<Clan> rivals = getRivals(year);
		int roll = Dice.randomInt(rivals.size() + 1);
		if (roll > rivals.size() - 1) {
			return randomClan(year, homeClan);
		}
		return rivals.get(roll);
	}
	
	public static Clan randomClan(int year, boolean homeClan) {
		ArrayList<Clan> list = new ArrayList<Clan>();
		for (Clan c : allClans.values()) {
			if (year > 3075 && homeClan != c.homeClan) {
				continue;
			}
			if (c.isActive(year)) {
				list.add(c);
			}
		}
		return list.get(Dice.randomInt(list.size()));
	}
	
	public static void loadClanData() {
		allClans = new HashMap<String, Clan>();
		File f = new File("data/clans.xml");
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(f);
		} catch (FileNotFoundException e) {
			System.err.println("Cannot find file clans.xml");
			return;
		}
		
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document doc = null;
		
		try {
			DocumentBuilder db = dbf.newDocumentBuilder();
			doc = db.parse(fis);
		} catch (Exception ex) {
			System.err.println(ex.getMessage());
		}
		
		Element clanElement = doc.getDocumentElement();
		NodeList nl = clanElement.getChildNodes();
		clanElement.normalize();
		
		for (int i = 0; i < nl.getLength(); i++) {
			Node wn = nl.item(i);
			if (wn.getNodeName().equalsIgnoreCase("clan")) {
				Clan c = loadFromXml(wn);
				allClans.put(c.code, c);
			}
		}
	}
	
	private static Clan loadFromXml(Node node) {
		Clan retVal = new Clan();
		
		retVal.code = node.getAttributes().getNamedItem("code").getTextContent().trim();
		if (null != node.getAttributes().getNamedItem("start")) {
			retVal.startDate = Integer.parseInt(node.getAttributes().getNamedItem("start").getTextContent().trim());
		}
		if (null != node.getAttributes().getNamedItem("end")) {
			retVal.endDate = Integer.parseInt(node.getAttributes().getNamedItem("end").getTextContent().trim());
		}
		NodeList nl = node.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			Node wn = nl.item(i);
			
			if (wn.getNodeName().equalsIgnoreCase("fullName")) {
				retVal.fullName = wn.getTextContent().trim(); 
			} else if (wn.getNodeName().equalsIgnoreCase("abjured")) {
				retVal.abjurationDate = Integer.parseInt(wn.getTextContent().trim());
			} else if (wn.getNodeName().equalsIgnoreCase("nameChange")) {
				int start = retVal.startDate;
				int end = retVal.endDate;
				if (null != wn.getAttributes().getNamedItem("start")) {
					start = Integer.parseInt(wn.getAttributes().getNamedItem("start").getTextContent().trim());
				}
				if (null != wn.getAttributes().getNamedItem("end")) {
					end = Integer.parseInt(wn.getAttributes().getNamedItem("end").getTextContent().trim());
				}
				retVal.nameChanges.add(new DatedRecord(start, end, wn.getTextContent().trim()));
			} else if (wn.getNodeName().equalsIgnoreCase("rivals")) {
				int start = retVal.startDate;
				int end = retVal.endDate;
				if (null != wn.getAttributes().getNamedItem("start")) {
					start = Integer.parseInt(wn.getAttributes().getNamedItem("start").getTextContent().trim());
				}
				if (null != wn.getAttributes().getNamedItem("end")) {
					end = Integer.parseInt(wn.getAttributes().getNamedItem("end").getTextContent().trim());
				}
				String[] rivals = wn.getTextContent().trim().split(",");
				for (String r : rivals) {
					retVal.rivals.add(new DatedRecord(start, end, r));
				}
			} else if (wn.getNodeName().equalsIgnoreCase("homeClan")) {
				retVal.homeClan = true;
			}
		}
		
		return retVal;
	}
}
